# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from iobase.xmldata import (
    XmlNode,
)
from ionas.backupsync.info import (
    ConnectionInfo,
    TargetInfo,
)


MAX_UPLOAD_SIZE = 5120  # MB


class WorkingFolderTargetInfo(TargetInfo):

    def __init__(self, target_xmlnode):
        super(WorkingFolderTargetInfo, self).__init__(
            target_xmlnode.get_value('type'),
            None,
        )

        # get parameters
        self._target_xmlnode = target_xmlnode
        self._email = target_xmlnode.get_value('email', default=None)
        self._password = target_xmlnode.get_value('password', default=None)
        self._cabinet_id = target_xmlnode.get_value('cabinet', default=None)
        self._drawer_name = target_xmlnode.get_value('drawer', default=None)
        self._max_upload_size = int(target_xmlnode.get_value('upload_max_size', default=MAX_UPLOAD_SIZE))

        # TODO: validate
        if self._max_upload_size > MAX_UPLOAD_SIZE:
            raise Exception('max upload size too large ({0})'.format(
                self._max_upload_size,
            ))

    @property
    def cabinet_id(self):
        return self._cabinet_id

    @property
    def drawer_name(self):
        return self._drawer_name

    @property
    def email(self):
        return self._email

    @property
    def password(self):
        return self._password

    @property
    def max_upload_size(self):
        return self._max_upload_size
