# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry, XmlNode
from fal_info import Results
from datetime import datetime
import time


class Fxwf(Command):
    try:
        from fal_info import FAL_FXWF_PATH as COMMAND
    except:
        COMMAND_NAME = "fal_fxwf.py"

    ##########################################################################
    # 公開用
    def add(self, job, messages={}, log=True):
        default_messages = {
            Results.OK: ["MSG_FXWF_ADD", {}],
            None: ["ERR_FXWF_ADD", {}],
        }
        default_messages.update(messages)

        try:
            entry = self._add_modify(job, mode="add")
            result = entry.get_result() if isinstance(entry, Entry) else None

            message = default_messages.get(result, default_messages[None])
            self._set_message(message[0], arglist=message[1], log=log)

            return result in [Results.OK]
        except:
            return False

    def modify(self, job, messages={}, log=True):
        default_messages = {
            Results.OK: ["MSG_FXWF_MODIFY", {}],
            None: ["ERR_FXWF_MODIFY", {}],
        }
        default_messages.update(messages)

        try:
            entry = self._add_modify(job, mode="modify")
            result = entry.get_result() if isinstance(entry, Entry) else None

            message = default_messages.get(result, default_messages[None])
            self._set_message(message[0], arglist=message[1], log=log)

            return result in [Results.OK]
        except:
            return False

    def delete(self, sid, messages={}, log=True):
        default_messages = {
            Results.OK: ["MSG_FXWF_DELETE", {}],
            None: ["ERR_FXWF_DELETE", {}],
        }
        default_messages.update(messages)

        try:
            entry = self._delete(sid)
            result = entry.get_result() if isinstance(entry, Entry) else None

            message = default_messages.get(result, default_messages[None])
            self._set_message(message[0], arglist=message[1], log=log)

            return result in [Results.OK]
        except:
            return False

    def list(self, uid=None, passwd=None, cabinet=None):

        def _entry_to_list(entry):
            try:
                drawerlist = []
                for drawer_node in entry.get().get_node_list("fxwfsync/drawerlist/drawer"):
                    try:
                        drawerlist.append({
                            "name": drawer_node.get_node("name").get_text(),
                            "desc": drawer_node.get_value("desc", ""),
                        })
                    except:
                        pass
                return drawerlist
            except:
                return []

        entry = self._list(uid=uid, passwd=passwd, cabinet=cabinet)
        return _entry_to_list(entry)

    def load(self):

        def _dict(entry):
            data = {}
            try:
                def __is_enabled(node, xpath=".", default=False):
                    try:
                        return node.get_node(xpath).is_enabled()
                    except:
                        return default

                for node in entry.get().get_node_list("sharelist/share"):
                    try:
                        sid = node.get_attr_value("id")
                        data[sid] = {}
                        data[sid]["sid"] = sid
                        data[sid]["enable"] = __is_enabled(
                                node, "shareservice/fxwfsync")
                        data[sid]["downloadinterval"] = int(node.get_value(
                                "shareservice/fxwfsync/downloadinterval", "0"))
                        data[sid]["uploadmaxfilesize"] = int(node.get_value(
                                "shareservice/fxwfsync/uploadmaxfilesize", "0"))
                        data[sid]["share"] = {}
                        data[sid]["share"]["name"] = node.get_value(
                                "name", "")
                        data[sid]["fxwf"] = {}
                        data[sid]["fxwf"]["cabinet"] = node.get_value(
                                "shareservice/fxwfsync/cabinetnumber", "")
                        data[sid]["fxwf"]["userid"] = node.get_value(
                                "shareservice/fxwfsync/userid", "")
                        data[sid]["fxwf"]["drawer"] = {}
                        data[sid]["fxwf"]["drawer"]["name"] = node.get_value(
                                "shareservice/fxwfsync/drawer/name", "")
                        #data[sid]["fxwf"]["drawer"]["desc"] = node.get_value(
                        #        "shareservice/fxwfsync/drawer/desc")
                    except:
                        continue
            except:
                pass
            return data

        entry = self._load()
        return _dict(entry)

    def query(self, sid):

        def _dict(entry):
            d = {}
            try:

                def _is_enabled(node, xpath=".", default=False):
                    try:
                        return node.get_node(xpath).is_enabled()
                    except:
                        return default

                fxwf_node = entry.get()
                d["sid"] = fxwf_node.get_node("share").get_attr_value("id")
                d["enable"] = _is_enabled(fxwf_node, "share/shareservice/fxwfsync")
                d["status"] = fxwf_node.get_value(
                        "share/shareservice/fxwfsync/status", "fail")
                d["downloadinterval"] = int(fxwf_node.get_value(
                        "share/shareservice/fxwfsync/downloadinterval", "0"))
                d["uploadmaxfilesize"] = int(fxwf_node.get_value(
                        "share/shareservice/fxwfsync/uploadmaxfilesize", "0"))
                d["share"] = {}
                d["share"]["name"] = fxwf_node.get_value(
                        "share/name", "")
                d["fxwf"] = {}
                d["fxwf"]["cabinet"] = fxwf_node.get_value(
                        "share/shareservice/fxwfsync/cabinetnumber", "")
                d["fxwf"]["uid"] = fxwf_node.get_value(
                        "share/shareservice/fxwfsync/userid", "")
                d["fxwf"]["passwd"] = fxwf_node.get_value(
                        "share/shareservice/fxwfsync/passwd", "")
                d["fxwf"]["drawer"] = {}
                d["fxwf"]["drawer"]["name"] = fxwf_node.get_value(
                        "share/shareservice/fxwfsync/drawer/name", "")
                d["fxwf"]["drawer"]["desc"] = fxwf_node.get_value(
                        "share/shareservice/fxwfsync/drawer/desc", "")
            except:
                pass
            return d

        entry = self._query(sid)
        return _dict(entry)

    def download(self, sid, messages={}, log=True):
        #default_messages = {
        #    Results.OK: ["MSG_OK", {}],  # FIXME msg
        #    None: ["MSG_NG", {}],  # FIXME msg
        #}
        #default_messages.update(messages)

        entry = self._download(sid)
        result = entry.get_result() if isinstance(entry, Entry) else None

        #message = default_messages.get(result, default_messages[None])
        #self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def dlmode(self, enable=None, log=True):

        code = None
        if isinstance(enable, bool):
            try:
                entry = Entry('save', self._load().get())
                try:
                    entry.get().get_node('dlmode')
                except:
                    entry.get().append_child('dlmode')
                entry.get().get_node('dlmode').enable(enable)
                self.request(entry)
                if (enable):
                    code = 'MSG_FXWF_SYNC_DIRECTION_ENABLE'
                else:
                    code = 'MSG_FXWF_SYNC_DIRECTION_DISABLE'
            except:
                code = 'ERR_FXWF_SYNC_DIRECTION_MODIFY'
                return None

        try:
            enable = self._load().get().get_node('dlmode').is_enabled()
        except:
            enable = False

        if code:
            self._set_message(code, arglist={}, log=log)

        return enable

                                                                             #
    ##########################################################################

    ##########################################################################
    # 内部用
    def _add_modify(self, job, mode="modify"):
        """
        job = {
            "sid": "123456789",  # NOTE modify only
            "enable": True,
            "downloadinterval": 3600,
            "uploadmaxfilesize": 10,
            "share": {
                "name": "disk1",  # NOTE add only
            },
            "chbox2": {
                "portal": "MyPortal",
                "loginid": "abc",
                "passwd": "pa55w0rd",
                "folder": {
                    "path": "/Users/MyFolder",
                },
            },
        }
        return Entry or None
        """
        entry = Entry(
                mode,
                """<fxwf><share><shareservice><fxwfsync /></shareservice></share></fxwf>""")
        try:
            """
            <chbox2>
                <share id="1234567890">
                    <name>disk1</name>
                    <shareservice>
                        <chbox2sync enable="true">
                            <portalid>1234567890</portalid>
                            <loginid>qwertyuiop</loginid>
                            <passwd>pa55w0rd</passwd>
                            <folder>
                                <path>/Users/MyFolder</path>
                            </folder>
                            <downloadinterval>3600</downloadinterval>
                            <uploadmaxsize>10</uploadmaxsize>
                        </chbox2>
                    </shareservice>
                </share>
            </chbox2>
            """
            share_node = entry.get().get_node("share")
            if mode == "modify":
                share_node.set_attrs({"id": job["sid"]})
            if mode == "add":
                share_node.append_child("name", text=job["share"]["name"])
            fxwf_node = share_node.get_node("shareservice/fxwfsync")
            fxwf_node.enable(job["enable"])
            if job["enable"]:
                fxwf_node.append_child("userid", text=job["fxwf"]["uid"])
                fxwf_node.append_child("passwd", text=job["fxwf"]["passwd"])
                fxwf_node.append_child("cabinetnumber", text=job["fxwf"]["cabinet"])
                fxwf_node.append_child("drawer")
                fxwf_node.get_node("drawer").append_child("name", text=job["fxwf"]["drawer"]["name"])
                if isinstance(job["fxwf"]["drawer"]["desc"], (str, unicode)):
                    fxwf_node.get_node("drawer").append_child("desc", text=job["fxwf"]["drawer"]["desc"])
                fxwf_node.append_child("downloadinterval", text=str(job["downloadinterval"]))
                fxwf_node.append_child("uploadmaxfilesize", text=str(job["uploadmaxfilesize"]))
        except KeyError:
            return None

        return self.request(entry)

    def _delete(self, sid):
        entry = Entry("delete", """<fxwf><share /></fxwf>""")
        entry.get().get_node("share").set_attrs({"id": sid})
        entry = self.request(entry)
        return entry

    def _list(self, uid=None, passwd=None, cabinet=None):
        entry = Entry(u"list", u"""<fxwf><fxwfsync /></fxwf>""")
        fxwfsync_node = entry.get().get_node(u"fxwfsync")
        fxwfsync_node.append_child(u"userid", text=uid)
        fxwfsync_node.append_child(u"passwd", text=passwd)
        fxwfsync_node.append_child(u"cabinetnumber", text=cabinet)
        return self.request(entry)

    def _load(self):
        entry = Entry("load", """<fxwf />""")
        return self.request(entry)

    def _query(self, sid):
        entry = Entry("query", """<fxwf><share /></fxwf>""")
        entry.get().get_node("share").set_attrs({"id": sid})
        entry = self.request(entry)
        return entry

    def _download(self, sid):
        entry = Entry("download", """<fxwf><share /></fxwf>""")
        entry.get().get_node("share").set_attrs({"id": sid})
        entry = self.request(entry)
        return entry

                                                                             #
    ##########################################################################


if __name__ == '__main__':
    fxwf = Fxwf()

    for drawer in fxwf.list(cabinet="", uid="", passwd=""):
        print drawer

